<?php

/**
 * @file
 * Hooks and preprocess functions for the Blazy module.
 */

use Drupal\blazy\Blazy;

/**
 * Returns HTML for a elevatezoomplus.
 *
 * @param array $variables
 *   An associative array containing:
 *   - element: The original theme elements.
 *   - settings: HTML related settings.
 *   - stage: The static main preview image if not using Slick with asNavFor.
 *
 * @ingroup themeable
 */
function theme_elevatezoomplus(array $variables) {
  extract($variables);

  // Only provides static stage if not using Slick asNavFor.
  $stage = empty($stage) ? '' : drupal_render($stage);

  // Appends and wraps the already-rendered #children.
  return '<div' . drupal_attributes($attributes_array) . '>' . $stage . $content . '</div>';
}

/**
 * Prepares variables for theme_elevatezoomplus().
 */
function template_preprocess_elevatezoomplus(array &$variables) {
  // Supported themes: Blazy (field|item_list), slick_wrapper, gridstack.
  $element = $variables['element'];
  $variables['settings'] = [];
  foreach (['blazy', 'build', 'context', 'settings'] as $key) {
    if (isset($element['#' . $key])) {
      $settings = isset($element['#' . $key]['settings']) ? $element['#' . $key]['settings'] : [];
      $variables['settings'] = ($key == 'build' || $key == 'context') ? $settings : $element['#' . $key];
      break;
    }
  }

  // Reset theme attributes as we are a wrapper/ container here.
  $settings = $variables['settings'];
  $json = elevatezoomplus()->getOptions($settings);
  $variables['attributes_array'] = ['data-elevatezoomplus' => drupal_json_encode($json)];
  $variables['attributes_array']['class'][] = 'elevatezoomplus-wrapper';

  // Checks if we have Slick asNavFor, or regular Slick, Blazy Grid, etc.
  $variables['stage'] = empty($settings['nav']) ? elevatezoomplus_build($settings) : [];
  $variables['content'] = isset($element['#children']) ? $element['#children'] : [];
}

/**
 * Build the main gallery image/ stage, if not using Slick asNavFor.
 */
function elevatezoomplus_build(array $settings = []) {
  $uri   = $settings['first_uri'];
  $item  = isset($settings['first_item']) ? $settings['first_item'] : new \stdClass();
  $style = empty($settings['image_style']) ? '' : $settings['image_style'];
  $style = empty($settings['box_style']) ? $style : $settings['box_style'];
  $url   = empty($settings['box_style']) ? file_create_url($uri) : image_style_url($settings['box_style'], $uri);
  $style = empty($settings['thumbnail_style']) ? $style : $settings['thumbnail_style'];
  $param = ['image_style' => $style, 'lazy' => 'blazy', 'uri' => $uri];
  $attrs = ['class' => ['elevatezoomplus'], 'data-zoom-image' => $url];
  $build = ['item' => $item, 'settings' => $param, 'item_attributes' => $attrs];
  $stage = blazy()->getBlazy($build);

  $stage['#attached'] = blazy()->attach($param + $settings);

  $attrs = ['class' => ['elevatezoomplus-stage blazy'], 'data-blazy' => ''];
  return Blazy::container($stage, $attrs);
}
