/**
 * @file
 * Provides ElevateZoomPlus loader.
 */

(function ($, Drupal, drupalSettings) {

  'use strict';

  /**
   * ElevateZoomPlus public methods.
   *
   * @namespace
   */
  Drupal.elevateZoomPlus = Drupal.elevateZoomPlus || {
    $instance: null,
    $el: null,
    $wrapper: null,
    $slider: null,
    itemSelector: '.elevatezoomplus',
    options: {},
    dataConfig: {},
    attach: function () {
      var me = this;
      var options = drupalSettings.elevateZoomPlus || {};
      var hasMedia = $('.media--video', me.$wrapper).length;
      var $static = $('[data-elevatezoomplus-gallery]', me.$wrapper);

      me.options = $.extend({}, options, me.dataConfig);
      me.$slider = $('.slick--main > .slick__slider', me.$wrapper);

      // Provides static gallery if not Slick Carousel: Blazy Grid, Gridstack.
      if (!me.$slider.length && $static.length) {
        if (!$static.attr('id')) {
          me.options.gallerySelector = '[data-elevatezoomplus-gallery]';
        }
        else {
          me.options.gallery = $static.attr('id');
        }
      }

      if (hasMedia && Drupal.blazyBox) {
        Drupal.blazyBox.attach();
      }

      me.$wrapper.addClass('elevatezoomplus-wrapper--on');
    },

    init: function () {
      var me = Drupal.elevateZoomPlus;

      // Needs to rebuild the main preview image for each slide.
      me.$el = $(me.$slider.length ? '.slick-current ' + me.itemSelector : me.itemSelector, me.$wrapper);
      me.$el.ezPlus(me.options);
      me.$instance = me.$el.data('ezPlus');
    }
  };

  /**
   * ElevateZoomPlus gallery item functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .elevatezoomplus-wrapper HTML element.
   */
  function doZoomWrapper(i, elm) {
    var me = Drupal.elevateZoomPlus;
    var $elm = $(elm);

    me.dataConfig = $elm.data('elevatezoomplus') || {};

    // Bail out if we have no config available.
    if (!me.dataConfig) {
      return;
    }

    // Attach our behaviors.
    me.$wrapper = $elm;
    me.attach();

    // Integrates with Slick with asNavFor.
    if (me.$slider.length) {
      me.$slider.on('init.ez', me.init);
      me.$slider.on('afterChange.ez', me.init);
    }
    else {
      // Integrates with Slick without asNavFor, Blazy Grid, Gridstack, etc.
      me.init();
    }
  }

  /**
   * Triggers video zoom on a click event.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The [data-elevatezoomplus-gallery] HTML element.
   */
  function doZoomGallery(i, elm) {
    var $elm = $(elm);

    /**
     * Triggers video zoom on a click event.
     */
    function triggerZoomVideo() {
      var $item = $(this);
      var media = $item.data('media') || false;
      var video = media && media.type === 'video';

      // Build own video fullscreen as ElevateZoomPlus is not for video.
      if (video && Drupal.blazyBox) {
        Drupal.blazyBox.open($item.parent().attr('href'));
      }
    }

    $elm.on('click', '.media--switch--elevatezoomplus.media--video', triggerZoomVideo);
  }

  /**
   * Attaches ElevateZoomPlus behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.elevateZoomPlus = {
    attach: function (context) {
      // Hooks into [data-blazy] to make it available for all formatters.
      // This is okay as this script is conditionally loaded per formatter.
      $('.elevatezoomplus-wrapper', context).once('elevatezoomplus-wrapper', doZoomWrapper);
      $('[data-elevatezoomplus-gallery]', context).once('elevatezoomplus-gallery', doZoomGallery);
    }
  };

}(jQuery, Drupal, Drupal.settings));
